/**
 * @fileoverview feature sliced relative path checker
 * @author Alexander
 */
"use strict";

const path = require('path');

//------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------

/** @type {import('eslint').Rule.RuleModule} */
module.exports = {
  meta: {
    type: null, // `problem`, `suggestion`, or `layout`
    docs: {
      description: "feature sliced relative path checker",
      recommended: false,
      url: null, // URL to the documentation page for this rule
    },
    fixable: null, // Or `code` or `whitespace`
    schema: [], // Add a schema if the rule has options
  },

  create(context) {
    return {
      ImportDeclaration(node) {
        // example: "app/entities/Article"
        const importTo = node.source.value;

        // current full path in mac OS: /Users/aleksandr/Desktop/Component.tsx
        const fromFilename = context.filename;

        if(shouldBeRelative(fromFilename, importTo)) {
          context.report(node, "Within one slice, all paths must be relative")
        }
      }
    };
  },
};

const layers = {
  'entities': 'entities',
  'features': 'features',
  'shared': 'shared',
  'pages': 'pages',
  'widgets': 'widgets'

}

function isPathRelative(path) {
  return path === "." || path.startsWith('./') || path.startsWith("../")
}

// from - текущее местоположение в проекте
// to - проверяемый импорт
function shouldBeRelative(from, to) {
  // возвращает true - если путь должен быть относительным
  // false - если путь должен быть абсолютным

  if(isPathRelative(to)) {
    return false
  }

  // to ===> // example: "app/entities/Article" ЧТО ИМПОРТИРУЕМ
  // from ===> current full path in mac OS: /Users/aleksandr/Desktop/Component.tsx ОТКУДА ИМПОРТИРУЕМ

  const toArray = to.split("/");

  const toLayer = toArray[0]; // entities
  const toSlice = toArray[1]; // Article

  if(!toLayer || !toSlice || !layers[toLayer]) {
    return false;
  }

  const fromNormalizedPath = path.toNamespacedPath(from);
  const isWindowsOS = fromNormalizedPath.includes('\\');
  const fromPath = fromNormalizedPath.split('src')[1];
  const fromArray = fromPath.split(isWindowsOS ? '\\' : '/'); // [ '', 'entities', 'Article' ]

  const fromLayer = fromArray[1];
  const fromSlice = fromArray[2];

  if(!fromLayer || !fromSlice || !layers[fromLayer]) {
    return false;
  }

  return fromSlice === toSlice && toLayer === fromLayer;
}
